//
//  CoreDataStack.swift
//  BookList
//
//  Created by Natsuko Nishikata on 2015/12/09.
//  Copyright © 2015年 Natsuko Nishikata. All rights reserved.
//

import Foundation
import CoreData

class CoreDataStack {
    
    let context:NSManagedObjectContext
    
    // ドキュメントディレクトリのURL
    let appDocumentDirURL: NSURL = {
        let fileManager = NSFileManager.defaultManager()
        let urls = fileManager.URLsForDirectory(.DocumentDirectory,
            inDomains:.UserDomainMask)
        return urls.last!
    }()
    
    init() {
        // データモデルファイルのURL --- (1-1)
        let bundle = NSBundle.mainBundle()
        guard let modelURL =
            bundle.URLForResource("BookListModel", withExtension: "momd") else {
                fatalError()
        }
        
        // URLを指定して管理オブジェクトモデルを生成 --- (1-2)
        guard let model = NSManagedObjectModel(contentsOfURL: modelURL) else {
            fatalError()
        }
        
        // 管理オブジェクトモデルを指定して永続ストアコーディネータを生成 --- (2)
        let coordinator = NSPersistentStoreCoordinator(managedObjectModel: model)
        
        // 管理オブジェクトコンテキストを生成（メインスレッド）
        // 永続ストアコーディネータと関連付け --- (3)
        context = NSManagedObjectContext(concurrencyType: .MainQueueConcurrencyType)
        context.persistentStoreCoordinator = coordinator
    }
  
    // 永続ストアの追加
    func addPersistentStoreWithCompletionHandler(completionHandler: (()->Void)?) {
        // 永続ストアの追加（時間がかかることがあるのでGCDを使う）
        let backgroundQueue =
        dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_BACKGROUND, 0)
        
        dispatch_async(backgroundQueue) {
            
            let dirURL = self.appDocumentDirURL
            let storeURL = dirURL.URLByAppendingPathComponent("BookList.sqlite")
            
            let options = [NSMigratePersistentStoresAutomaticallyOption: true,
                           NSInferMappingModelAutomaticallyOption: true]

            do {
                // 永続ストアコーディネータに永続ストアを追加
                let coordinator = self.context.persistentStoreCoordinator!
                try coordinator.addPersistentStoreWithType(NSSQLiteStoreType,
                    configuration: nil,
                    URL: storeURL,
                    options: options)
                // ストアの追加完了を通知
                completionHandler?()
            } catch let error as NSError {
                fatalError("\(error)")
            }
        }
    }
    
    // 管理オブジェクトコンテキストの保存
    func saveContext () throws {
        // コンテキストに変更があるかを確認
        // 保存処理はコンテキストに変更がある場合のみ行う
        if context.hasChanges {
            do {
                // コンテキストを保存
                try context.save()
            } catch let error as NSError {
                print("\(error)")
                throw error
            }
        }
    }
}
