//
//  DynamicTableViewController.swift
//  TableViewSample
//
//  Created by Natsuko Nishikata on 2015/12/08.
//  Copyright © 2015年 Natsuko Nishikata. All rights reserved.
//

import UIKit

class DynamicTableViewController: UITableViewController {
    
    var fruits:[(name:String, color:String)] =
    [   ("Apple", "Red"),
        ("Banana", "Yellow"),
        ("Grape", "Purple"),
        ("Mandarin", "Orange"),
        ("Strawberry", "Red"),
        ("Blueberry", "Purple"),
    ]

    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.navigationItem.rightBarButtonItem = self.editButtonItem()
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

    // MARK: - Table view data source

    // セクション数
    override
    func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        return 1
    }
    
    // 行数
    override
    func tableView(tableView: UITableView,
        numberOfRowsInSection section: Int) -> Int {
            return fruits.count // fruitsの要素数がテーブルの行数になる
    }
    
    // セクション名
    override
    func tableView(tableView: UITableView,
        titleForHeaderInSection section: Int) -> String? {
            return "fruits"
    }
    
    // セル
    override
    func tableView(tableView: UITableView,
        cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
            // 再利用識別子"MyCell"を持つセルを再利用キューから取り出す
            let cell = tableView.dequeueReusableCellWithIdentifier("MyCell",
                forIndexPath: indexPath)
            
            let fruit = fruits[indexPath.row] // indexPath.rowが行番号を示す
            
            // 標準セルのUILabelに値をセット
            cell.textLabel!.text = fruit.name
            cell.detailTextLabel!.text = fruit.color
            
            return cell
    }
    
    // テーブルビューの編集を許可
    override
    func tableView(tableView: UITableView,
        canEditRowAtIndexPath indexPath: NSIndexPath) -> Bool {
            return true
    }
    
    // 編集操作が確定したときの処理
    override
    func tableView(tableView: UITableView,
        commitEditingStyle editingStyle: UITableViewCellEditingStyle,
        forRowAtIndexPath indexPath: NSIndexPath) {
            if editingStyle == .Delete { // セルの削除
                // データソースを更新
                fruits.removeAtIndex(indexPath.row)
                // テーブルビューを更新（行を削除）
                tableView.deleteRowsAtIndexPaths([indexPath],
                    withRowAnimation: .Fade)
            } else if editingStyle == .Insert {
                // セル挿入時の処理
            }    
    }
    
    // セルの移動を許可
    override
    func tableView(tableView: UITableView,
        canMoveRowAtIndexPath indexPath: NSIndexPath) -> Bool {
            return true
    }
    
    // 移動処理
    override
    func tableView(tableView: UITableView,
        moveRowAtIndexPath fromIndexPath: NSIndexPath,
        toIndexPath: NSIndexPath) {
            // データソースを更新
            let fruit = fruits[fromIndexPath.row]
            fruits.removeAtIndex(fromIndexPath.row)
            fruits.insert(fruit, atIndex: toIndexPath.row)
    }
    

    // MARK: - Navigation

    override
    func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        if segue.identifier == "ShowDetail" {
            // セグエの起点がsenderとなる
            let cell = sender as! UITableViewCell
            // インデックスパスを取得
            let indexPath = tableView.indexPathForCell(cell)!
            // 遷移先ビューコントローラ
            let vc =
            segue.destinationViewController as! StaticTableViewController
            // 表示対象となるデータソースをセット
            vc.fruit = fruits[indexPath.row]
        }
    }

}
