//
//  BookListTests.swift
//  BookListTests
//
//  Created by Natsuko Nishikata on 2015/12/16.
//  Copyright © 2015年 Natsuko Nishikata. All rights reserved.
//

import XCTest
import CoreData
@testable import BookList

class BookListTests: XCTestCase {
    
    var coreDataStack: TestCoreDataStack!
    
    override func setUp() {
        super.setUp()
        coreDataStack = TestCoreDataStack()
    }
    
    override func tearDown() {
        coreDataStack = nil
        super.tearDown()
    }
    
    func testAddBook() {
        let book = NSEntityDescription.insertNewObjectForEntityForName("Book",
            inManagedObjectContext: coreDataStack.context) as! Book
        book.title = "Book Title"
        do {
            try coreDataStack.saveContext()
        } catch {
            // エラーになってはいけない
            XCTFail()
        }
        XCTAssertNotNil(book)                     // bookが生成できている
        XCTAssertTrue(book.title == "Book Title") // タイトルが正しくセットされている
    }

    func testAddBookWithoutTitle() {
        let book = NSEntityDescription.insertNewObjectForEntityForName("Book",
            inManagedObjectContext: coreDataStack.context) as! Book
        do {
            try coreDataStack.saveContext()
        } catch let error as NSError {
            // 必須属性に値を入れていないためエラーが投げられる
            XCTAssertNotNil(error)
            
            // エラーが正しくセットされているかを確認（システム検証エラー）
            XCTAssertEqual(error.code, NSValidationMissingMandatoryPropertyError)
        }
        
        XCTAssertNotNil(book)
        XCTAssertNil(book.title)
    }
    
    func testAddBookWhiteSpaceTitle() {
        let book = NSEntityDescription.insertNewObjectForEntityForName("Book",
            inManagedObjectContext: coreDataStack.context) as! Book
        book.title = "     "
        do {
            try coreDataStack.saveContext()
        } catch let error as NSError {
            // カスタム検証メソッドによりエラーが投げられる
            XCTAssertNotNil(error)
            
            // エラーが正しくセットされているかを確認（カスタム検証エラー）
            XCTAssertEqual(error.code, BookErrorCode.InvalidTitle.rawValue)
        }
        
        XCTAssertNotNil(book)
        XCTAssertTrue(book.title == "     ")
    }
    
    func testAddBookWishAndNoURL() {
        let book = NSEntityDescription.insertNewObjectForEntityForName("Book",
            inManagedObjectContext: coreDataStack.context) as! Book
        book.title = "title"
        book.wish = true
        book.url = nil
        do {
            try coreDataStack.saveContext()
        } catch let error as NSError {
            // カスタム検証メソッドによりエラーが投げられる
            XCTAssertNotNil(error)
            
            // エラーが正しくセットされているかを確認（カスタム検証エラー）
            XCTAssertEqual(error.code, BookErrorCode.NoURL.rawValue)
        }
        
        XCTAssertNotNil(book)
    }
}
